/**
 * OW2 Util
 * Copyright (C) 2010-2011 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: SftpTransferProvider.java 5702 2011-01-29 18:17:28Z loris $
 * --------------------------------------------------------------------------
 */
package org.ow2.util.protocol.ssh.impl.jsch;

import java.io.File;
import java.util.HashMap;

import org.ow2.util.file.FileUtils;
import org.ow2.util.protocol.api.ITransferProvider;
import org.ow2.util.protocol.api.ProtocolException;
import org.ow2.util.protocol.ssh.api.SshProtocolConfig;

import com.jcraft.jsch.ChannelSftp;
import com.jcraft.jsch.JSchException;
import com.jcraft.jsch.SftpException;

/**
 *
 * @author Loris Bouzonnet
 */
public class SftpTransferProvider implements ITransferProvider {

	private final SshProtocolSession protocolSession;

	public SftpTransferProvider(final SshProtocolSession session) {
		this.protocolSession = session;
	}

	public void download(final String source, final String destination, boolean recursive)
			throws ProtocolException {

		final String rfile;
		final SshProtocolSession protocolSession;

		if (this.protocolSession == null) {
			final String url = source.substring(0, source.indexOf("/"));
			rfile =
				source.substring(source.indexOf("/"));
			protocolSession = new SshProtocol().createSession(
					SshProtocolConfigFactory.createSshProtocolConfig(url, null));
		} else {
			rfile = source;
			protocolSession = this.protocolSession;
		}

		ChannelSftp channel = null;
		try {
			channel =
				(ChannelSftp) protocolSession.createJschChannel("sftp", null);
			channel.connect();
			channel.get(rfile, destination);
		} catch (final SftpException e) {
			throw new ProtocolException(e);
		} catch (final JSchException e) {
			throw new ProtocolException(e);
		} finally {
			if (channel != null) {
				channel.disconnect();
			}
			if (this.protocolSession == null) {
				FileUtils.close(protocolSession);
			}
		}
	}

	public void upload(final String source, final String destination, boolean recursive)
			throws ProtocolException {

		final String rfile;
		final SshProtocolSession protocolSession;

		if (this.protocolSession == null) {
			final String url = destination.substring(0, destination.indexOf("/"));
			rfile =
				destination.substring(destination.indexOf("/"));
			protocolSession = new SshProtocol().createSession(
					SshProtocolConfigFactory.createSshProtocolConfig(url, null));
		} else {
			rfile = destination;
			protocolSession = this.protocolSession;
		}

		ChannelSftp channel = null;
		try {
			channel =
				(ChannelSftp) protocolSession.createJschChannel("sftp", null);
			channel.connect();
			channel.put(source, rfile);
		} catch (final SftpException e) {
			throw new ProtocolException(e);
		} catch (final JSchException e) {
			throw new ProtocolException(e);
		} finally {
			if (channel != null) {
				channel.disconnect();
			}
			if (this.protocolSession == null) {
				FileUtils.close(protocolSession);
			}
		}
	}

	public static void main(final String[] arg) throws ProtocolException {

		if (arg.length!=2){
			throw new ProtocolException();
		}

		final String url;
		String downloadFile = null;
		String uploadFile = null;

		if (arg[0].contains(":")) {
			url = arg[0].substring(0, arg[0].indexOf("/"));
			downloadFile = arg[0].substring(arg[0].indexOf("/"));
		} else if (arg[1].contains(":")) {
			url = arg[1].substring(0, arg[1].indexOf("/"));
			uploadFile = arg[1].substring(arg[1].indexOf("/"));
		} else {
			throw new ProtocolException();
		}

		final HashMap<String, String> context = new HashMap<String, String>();
		context.put(SshProtocolConfigFactory.PRIVATE_KEY_URL,
				System.getProperty("user.home") + File.separator + ".ssh" + File.separator + "id_rsa");
		context.put(SshProtocolConfigFactory.KNOWN_HOSTS,
				System.getProperty("user.home") + File.separator + ".ssh" + File.separator + "known_hosts");

		final SshProtocolConfig protocolConfig =
			SshProtocolConfigFactory.createSshProtocolConfig(url, context);

		final SshProtocolSession session = new SshProtocol().createSession(protocolConfig);
		final SftpTransferProvider sftpTransfertProvider = new SftpTransferProvider(session);

		if (downloadFile != null) {
			sftpTransfertProvider.download(downloadFile, arg[1], false);
		} else {
			sftpTransfertProvider.upload(arg[0], uploadFile, false);
		}

		FileUtils.close(session);
	}

}
